/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
 
package org.ow2.easywsdl.tooling.xsd2xml.test;

import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Calendar;
import java.util.Map;

import javax.xml.namespace.QName;

import org.junit.Assert;
import junit.framework.TestCase;

import org.ow2.easywsdl.schema.DefaultSchemaImpl;
import org.ow2.easywsdl.schema.SchemaFactory;
import org.ow2.easywsdl.schema.api.Element;
import org.ow2.easywsdl.schema.api.Schema;
import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.api.Type;
import org.ow2.easywsdl.tooling.xsd2xml.XSD2XML;
import org.ow2.easywsdl.wsdl.WSDLFactory;
import org.ow2.easywsdl.wsdl.api.Description;

/**
 * @author Nicolas Salatge - EBM WebSourcing
 */
public class XSD2XMLTest extends TestCase {

	private String TARGET_NAMESPACE = "http://petals.ow2.org/";

	public QName cName(String name) {
		return new QName(TARGET_NAMESPACE, name);
	}

	/**
	 * @param name
	 */
	public XSD2XMLTest(final String name) {
		super(name);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see junit.framework.TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see junit.framework.TestCase#tearDown()
	 */
	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	public void testReaderSchema() throws URISyntaxException, SchemaException, IOException {
		final URL xsdUrl = Thread.currentThread().getContextClassLoader().getResource("descriptors/sample.xsd");
		final Schema desc = SchemaFactory.newInstance().newSchemaReader().read(xsdUrl);

		// assert on parent
		Assert.assertNotNull(desc);

		// assert on types
		Assert.assertEquals(5, desc.getTypes().size());
		Assert.assertEquals("PurchaseOrderType", desc.getTypes().get(0).getQName().getLocalPart());
		Assert.assertEquals("USAddress", desc.getTypes().get(1).getQName().getLocalPart());
		Assert.assertEquals("Items", desc.getTypes().get(2).getQName().getLocalPart());
		Assert.assertEquals("SKU", desc.getTypes().get(3).getQName().getLocalPart());

		// assert on elements
		Assert.assertEquals(2, desc.getElements().size());
		Assert.assertEquals("purchaseOrder", desc.getElements().get(0).getQName().getLocalPart());
		Assert.assertEquals("comment", desc.getElements().get(1).getQName().getLocalPart());

		//		System.out.println(desc);
		//		System.out.println(desc.getTypes().size());
		//		System.out.println(desc.getElements().size());
		//
		//
		//		System.out.println(XSD2XML.newInstance().printXml(desc.getElements().get(0),"?"));
		//		System.out.println(XSD2XML.newInstance().printXml(desc.getElements().get(1),"?"));

	}


	public void testImportSchema() throws URISyntaxException, SchemaException, IOException {

		final URL xsdUrl = Thread.currentThread().getContextClassLoader().getResource("descriptors/reservationList.xsd");
		final Schema desc = SchemaFactory.newInstance().newSchemaReader().read(xsdUrl);

		// assert on parent
		Assert.assertNotNull(desc);

		//		System.out.println(desc);
		//		System.out.println(desc.getTypes().size());
		//		System.out.println(desc.getElements().size());
		//
		//		for (final Element e : desc.getElements()) {
		//			System.out.println(XSD2XML.newInstance().printXml(e,"?"));
		//		}

		// System.out.println(desc.getElements().get(0).printXml("?"));
		// System.out.println(desc.getElements().get(1).printXml("?"));

	}


	public void testCreateElmtSchema() throws URISyntaxException, SchemaException {
		Schema schema = SchemaFactory.newInstance().newSchema();
		schema.setTargetNamespace("http://petals.ow2.org/");
		org.ow2.easywsdl.schema.api.Element elmt = schema.createElement();


		elmt.setQName(new QName("nbAssure"));
		elmt.setType(SchemaFactory.getDefaultSchema().getTypeInt());

		org.jdom.Element jDomElmt = XSD2XML.newInstance().generateElement(elmt, "?");
		Assert.assertNotNull(jDomElmt);

		// print msg
		//		System.out.println("Message: " + new XMLOutputter(Format.getPrettyFormat()).outputString(jDomElmt));

	}

	public void testCreateArraySchema() throws URISyntaxException, SchemaException, IOException {

		final URL xsdUrl = Thread.currentThread().getContextClassLoader().getResource("descriptors/ManagerProcess.xsd");
		final Schema desc = SchemaFactory.newInstance().newSchemaReader().read(xsdUrl);


		Element array = desc.createElement();
		array.setQName(new QName("http://irit.contractnet.org", "bidArray"));
		array.setType(desc.getTypes().get(0));

		//		System.out.println(XSD2XML.newInstance().printXml(array,"?"));

	}

	public void testCreateMiseAJourSuiviCollecteElement() throws URISyntaxException, SchemaException, IOException {

		final URL xsdUrl = Thread.currentThread().getContextClassLoader().getResource("descriptors/i_servicemajsuivi.xsd");
		final Schema desc = SchemaFactory.newInstance().newSchemaReader().read(xsdUrl);

		Element miseAJourSuiviCollecte = desc.getElement(new QName("http://suivi.architecture.cnav.fr/", "miseAJourSuiviCollecte"));
		Assert.assertNotNull(miseAJourSuiviCollecte);
		Assert.assertEquals("miseAJourSuiviCollecte",miseAJourSuiviCollecte.getQName().getLocalPart());

		//		System.out.println(XSD2XML.newInstance().printXml(miseAJourSuiviCollecte,"?"));

	}

	public void testRejectedBidElement() throws URISyntaxException, SchemaException, IOException {

		final URL xsdUrl = Thread.currentThread().getContextClassLoader().getResource("descriptors/ContractorProcessAbstract.xsd");
		final Schema desc = SchemaFactory.newInstance().newSchemaReader().read(xsdUrl);


		Element miseAJourSuiviCollecte = desc.getElement(new QName("http://irit.contractnet.org/contractor", "ReceiveBidRejectRequest"));
		Assert.assertNotNull(miseAJourSuiviCollecte);
		Assert.assertEquals("ReceiveBidRejectRequest",miseAJourSuiviCollecte.getQName().getLocalPart());

		//		System.out.println(XSD2XML.newInstance().printXml(miseAJourSuiviCollecte,"?"));

	}


	public void testCreateGetPresenceElement() throws URISyntaxException, SchemaException, IOException {

		final URL xsdUrl = Thread.currentThread().getContextClassLoader().getResource("descriptors/PresenceEnablerSEI-V1_0.xsd");
		final Schema desc = SchemaFactory.newInstance().newSchemaReader().read(xsdUrl);

		Element getPresenceCollecte = desc.getElement(new QName("http://com.orange.ft.enabler.sei/PresenceEnabler/V1_0/PresenceQuerySEI", "getPresence"));
		Assert.assertNotNull(getPresenceCollecte);
		Assert.assertEquals("getPresence",getPresenceCollecte.getQName().getLocalPart());

		System.out.println(XSD2XML.newInstance().printXml(getPresenceCollecte,"?"));

	}


	public void testCreateGetPresenceElementFromWsdl() throws URISyntaxException, SchemaException, IOException {

		final URL wsdlUrl = Thread.currentThread().getContextClassLoader().getResource("descriptors/PresenceEnablerSEI-V1_0.wsdl");
		final Description desc = WSDLFactory.newInstance().newWSDLReader().read(wsdlUrl);

		Element getPresenceCollecte = null;
		for(Schema s: desc.getTypes().getSchemas()) {
			getPresenceCollecte = s.getElement(new QName("http://com.orange.ft.enabler.sei/PresenceEnabler/V1_0/PresenceQuerySEI", "getPresence"));
			if(getPresenceCollecte != null) {
				break;
			}
		}

		Assert.assertNotNull(getPresenceCollecte);
		Assert.assertEquals("getPresence",getPresenceCollecte.getQName().getLocalPart());

		System.out.println(XSD2XML.newInstance().printXml(getPresenceCollecte,"?"));

	}


	public void testAddDefaultValue() throws URISyntaxException, SchemaException, IOException {
		
		final URL xsdUrl = Thread.currentThread().getContextClassLoader().getResource("descriptors/ContractorProcessAbstract.xsd");
		final Schema desc = SchemaFactory.newInstance().newSchemaReader().read(xsdUrl);


		Element miseAJourSuiviCollecte = desc.getElement(new QName("http://irit.contractnet.org/contractor", "ReceiveBidRejectRequest"));
		Assert.assertNotNull(miseAJourSuiviCollecte);

		Map<Type, Object> map = XSD2XML.createDefaultMap("?");
		map.put(SchemaFactory.getDefaultSchema().getTypeInt(), "0");
		map.put(SchemaFactory.getDefaultSchema().getTypeBoolean(), "true");
		map.put(SchemaFactory.getDefaultSchema().getTypeDateTime(), Calendar.getInstance().getTime().toString());
		

		System.out.println(XSD2XML.newInstance().printXml(miseAJourSuiviCollecte, map, true, true));



	}
	
	
	
public void testAddDefaultValueWithEmptyMap() throws URISyntaxException, SchemaException, IOException {
		
		final URL xsdUrl = Thread.currentThread().getContextClassLoader().getResource("descriptors/ContractorProcessAbstract.xsd");
		final Schema desc = SchemaFactory.newInstance().newSchemaReader().read(xsdUrl);


		Element miseAJourSuiviCollecte = desc.getElement(new QName("http://irit.contractnet.org/contractor", "ReceiveBidRejectRequest"));
		Assert.assertNotNull(miseAJourSuiviCollecte);



		System.out.println(XSD2XML.newInstance().printXml(miseAJourSuiviCollecte, null, true, true));



	}
	
	
	
	public void testBindResponseElementFromWsdl() throws URISyntaxException, SchemaException, IOException {

		final URL wsdlUrl = Thread.currentThread().getContextClassLoader().getResource("descriptors/prosodie/authorizationService.wsdl");
		final Description desc = WSDLFactory.newInstance().newWSDLReader().read(wsdlUrl);

		Element bindResponse = null;
		for(Schema s: desc.getTypes().getSchemas()) {
			bindResponse = s.getElement(new QName("http://contract.prosodie.com/AuthorizationService/1/", "bindResponse"));
			if(bindResponse != null) {
				break;
			}
		}

		Assert.assertNotNull(bindResponse);
		Assert.assertEquals("bindResponse", bindResponse.getQName().getLocalPart());

		System.out.println(XSD2XML.newInstance().printXml(bindResponse,"?"));

	}

}
